/*-----------------------------------------------------------------------
 *
 * File Name: hello.c
 *
 * Author: Ronald Garcia -- created Tue Nov 10 22:21:07 EST 1998
 *
 * DESCRIPTION
 *     This is a very very simplified example of using ORBit to do
 * CORBA. Both the server and client sides are implemented in the same
 * executable here (which is perfectly legal! ).  Hopefully this
 * provides an accessible entry into ORBit/CORBA use  -- which writing
 * it did for me.
 *
 * REVISION HISTORY
 *
 * $Log$
 *
 *-----------------------------------------------------------------------
 */


#include <stdio.h>

/* Include the ORBit orb header */
#include <orb/orbit.h>

/* include the header file generated by orbit-idl. This contains the
 * declarations of the client side functions and a few other things.  */
#include "hello.h"

/* For this example, I wrote the implementation of the hello interface
 * in a seperate file.  Since you need to reference
 * poa_hello_servant for the server side of the app, it's declared
 * extern in the following file
 */
#include "helloimp.h"

/* Since C does not have built-in exception support, the OMG defined 
 * the CORBA_exception data structure to throw and detect faults */
CORBA_Environment ev;

int main (int argc, char *argv[]) {

  hello hi_serv;	      /* The server-side hello object */
  hello hi_client;	      /* The client-side hello object */
  PortableServer_POA poa;     /* The Portable Object adaptor */

  CORBA_ORB orb=NULL;         /* This is our ORB */
  char *ior;		      /* The Interoperable Object Reference (IOR)
				 is one way a client can locate a server */
  

  /*
   * Begin Server-side Code
   *   In this section, we create a CORBA object upon which we can call
   *   methods.  In this example, the server and client are in the same
   *   executable, but later we'll see that not only can they exist 
   *   separately, but the client code and server code change very little.
   */

  /* Initialize the orb. The initialization routine checks the command
   * line parameters for directives that affect the orb */
  orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);

  /* The POA is the connection between our server object and the ORB */
  poa = (PortableServer_POA)
    CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);

  /* We activate the POA so that it will accept requests on its objects */
  PortableServer_POAManager_activate(
         PortableServer_POA__get_the_POAManager(poa, &ev), &ev);

  /* ORBit needs a chance to hook in implementation-specific hooks to the
   * servant structure.  That happens in this IDL-generated procedure */
   POA_hello__init(&poa_hello_servant, &ev);


   /* Here is where we create the CORBA object, using our servant
    * structure to specify where the object's methods are implemented
    */
   PortableServer_POA_activate_object(poa, &poa_hello_servant, &ev);

  /* Here we get a CORBA Object handle so we can get the IOR */
  hi_serv = PortableServer_POA_servant_to_reference(poa,          
	  &poa_hello_servant, &ev);

  /* Here we get the IOR for the hi object.  Our "client" will use
   * the IOR to find the server to connect to */
  ior = CORBA_ORB_object_to_string(orb, hi_serv, &ev);
      
  /* print out the IOR, just so you can see what it looks like */
  printf ("%s\n", ior);

  /*
   * Begin Client-side code 
   *   The code from here to the end manipulates our server object
   */

  /* Initialize the CORBA_Environment to prepare for exception handling */
  CORBA_exception_init(&ev);


  /* Here's where we create and use a client */

  /* Here, we translate the ior (generated above) back into an object 
   * that we can call methods on. */
  hi_client = CORBA_ORB_string_to_object(orb, ior, &ev);
  if (!hi_client) {
    printf("Cannot bind to %s\n", ior);
    exit(1);
  }

  /* Here we call the hello method on the hello object. */
  hello_hello(hi_client, &ev);

  /* We're done now. Do some cleanup */
  CORBA_free(ior);
  CORBA_Object_release(hi_client, &ev);
  CORBA_ORB_shutdown(orb, CORBA_FALSE, &ev);

  /* The following line is specific to the server - we no longer need
   * hi_serv so let it go */
  CORBA_Object_release(hi_serv, &ev);

}


