/*-----------------------------------------------------------------------
 *
 * File Name: helloimp.c
 *
 * Author: Ronald Garcia -- created Tue Nov 10 21:15:36 EST 1998
 *
 * Revision: $Id$
 *
 *-----------------------------------------------------------------------
 *
 * NAME
 * helloimp.c
 *
 * SYNOPSIS
 *     <insert synopsis>
 *
 * DESCRIPTION
 *     This is the implementation for the hello interface's hello 
 *     command.  I put it in a seperate file just for kicks.
 *
 * DIAGNOSTICS
 *
 *-----------------------------------------------------------------------
 *
 * REVISION HISTORY
 *
 * $Log$
 *
 *-----------------------------------------------------------------------
 */

#include <stdio.h>
#include <orb/orbit.h>
#include "hello.h"

/* helloimpl.h only references the hello_servant. You really don't
 * need it */
#include "helloimp.h"

/* this is the definition of the methods supported by this server. */

void hello_hello_server_side (PortableServer_Servant servant,
				CORBA_Environment *ev) {
  printf("Hello, World!\n");
}


/* POA stuffs
 *	The following code creates the structures that will connect
 * our hello interface and its methods (ie. the hello method) to the
 * Portable Object Adaptor (POA).  It is the role of the POA (as far
 * as I know at least) , on the server side, to find/instantiate the 
 * implementation of the interface and and subsequently pass that 
 * implementation the method call received from a client (for more
 * explanation, see the OMG web page (www.omg.org) 
 */

/* Portable Servant 
 *	As per the CORBA 2.2 standard, "A servant is a language-specific
 * entity that can incarnate a CORBA object. In C, a servant is
 * composed of a data structure that holds the state of the object
 * along with a collection of method functions that manipulate that
 * state in order to implement the CORBA object."
 *	In short, our portable servant points to the c implementations
 * of the interface methods declared in the IDL file.
 *	For more indepth -- and probably more correct, better, etc
 * -- explanation, refer to the CORBA 2.2 standard, chapter 19.26.4
 */


/* The servant base is our procedural equivalent of a base class in
 * C++. If our interface had been inherited from another interface,
 * here is where we would declare any parent. Insofar as I can tell,
 * epv (Entry Point Vector) and vepv structures are used to facilitate
 * interface inheritence. */
PortableServer_ServantBase__epv base_epv = {
  NULL,
  NULL,
  NULL
};


/* this struct is filled with function pointers for all the methods
 * in this interface.  They have to be in the same order that they are
 * declared in the idl file.  The first entry is NULL, but I'm not
 * sure if that is always the case. I believe it's reserved for 
 * "user data". You'll see that the function names listed here were
 * defined above.  These are pointers to them. */
POA_hello__epv hello_epv = {
  (void *) NULL,
  hello_hello_server_side
};

/* The servant contains a pointer to a vepv, which contains a pointer
 * to the epv. My limited understanding of this is explained above */
POA_hello__vepv poa_hello_vepv = { &base_epv, &hello_epv };
POA_hello poa_hello_servant = { NULL, &poa_hello_vepv };

